/* -*- mode: c -*- */

/* Copyright (C) 2004-2024 Alexander Chernov <cher@ejudge.ru> */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "ejudge/super_proto.h"

#include "ejudge/xalloc.h"

#include <stdio.h>

unsigned char const * const super_proto_error_messages[] =
{
  "no error",
  "error code 1",
  "not connected",
  "invalid file descriptors",
  "write to server failed",
  "invalid socket name",
  "system call failed",
  "connection refused",
  "read from server failed",
  "unexpected EOF from server",
  "protocol error",
  "userlist-server is down",
  "permission denied",
  "invalid contest",
  "IP-address is banned",
  "contest root_dir is not set",
  "file does not exist",
  "log file is redirected to /dev/null",
  "read error",
  "file format is invalid",
  "unexpected userlist-server error",
  "contest is already used",
  "another contest is edited in this session",
  "not implemented yet",
  "invalid parameter",
  "no contest is edited",
  "duplicated login name",
  "such problem already exists",
  "this problem is used as base problem",
  "parameter is out of range",
  "request disabled in slave mode",

  [SSERV_ERR_EMPTY_REPLY] = "Reply text is empty",
  [SSERV_ERR_INV_OPER] = "Invalid operation",
  [SSERV_ERR_INV_SID] = "Invalid session id",
  [SSERV_ERR_INV_CONTEST] = "Invalid contest id",
  [SSERV_ERR_PERM_DENIED] = "Permission denied",
  [SSERV_ERR_INTERNAL] = "Internal error",
  [SSERV_ERR_ALREADY_EDITED] = "Contest is already edited",
  [SSERV_ERR_NO_EDITED_CNTS] = "No contest is edited",
  [SSERV_ERR_INV_FIELD_ID] = "Invalid field ID",
  [SSERV_ERR_INV_VALUE] = "Invalid value",
  [SSERV_ERR_CONTEST_ALREADY_EXISTS] = "Contest with this ID already exists",
  [SSERV_ERR_CONTEST_ALREADY_EDITED] = "Contest is edited by another person",
  [SSERV_ERR_INV_LANG_ID] = "Invalid Lang ID",
  [SSERV_ERR_INV_PROB_ID] = "Invalid Prob ID",
  [SSERV_ERR_INV_PACKAGE] = "Invalid package",
  [SSERV_ERR_ITEM_EXISTS] = "Such item already exists",
  [SSERV_ERR_OPERATION_FAILED] = "System operation failed",
  [SSERV_ERR_INV_USER_ID] = "Invalid User ID",
  [SSERV_ERR_NO_CONNECTION] = "No connection to the database",
  [SSERV_ERR_DB_ERROR] = "Database error",
  [SSERV_ERR_UNSPEC_PASSWD1] = "Password 1 is not specified",
  [SSERV_ERR_UNSPEC_PASSWD2] = "Password 2 is not specified",
  [SSERV_ERR_INV_PASSWD1] = "Password 1 is invalid",
  [SSERV_ERR_INV_PASSWD2] = "Password 2 is invalid",
  [SSERV_ERR_PASSWDS_DIFFER] = "Passwords do not match each other",
  [SSERV_ERR_UNSPEC_LOGIN] = "Login is not specified",
  [SSERV_ERR_INV_GROUP_ID] = "Invalid group ID",
  [SSERV_ERR_INV_FIRST_SERIAL] = "Invalid first serial number",
  [SSERV_ERR_INV_LAST_SERIAL] = "Invalid last serial number",
  [SSERV_ERR_INV_RANGE] = "Invalid serial number range",
  [SSERV_ERR_INV_LOGIN_TEMPLATE] = "Invalid login template",
  [SSERV_ERR_INV_REG_PASSWORD_TEMPLATE] = "Invalid registration password template",
  [SSERV_ERR_INV_CNTS_PASSWORD_TEMPLATE] = "Invalid contest password template",
  [SSERV_ERR_INV_CNTS_NAME_TEMPLATE] = "Invalid name template",
  [SSERV_ERR_INV_CSV_FILE] = "Invalid CSV file",
  [SSERV_ERR_INV_CHARSET] = "Invalid charset",
  [SSERV_ERR_INV_SEPARATOR] = "Invalid field separator",
  [SSERV_ERR_DATA_READ_ONLY] = "Data is read-only",
  [SSERV_ERR_TOO_MANY_MEMBERS] = "Too many members",
  [SSERV_ERR_INV_SERIAL] = "Invalid member",
  [SSERV_ERR_INV_EMAIL] = "Invalid email",
  [SSERV_ERR_INV_GROUP_NAME] = "Invalid group name",
  [SSERV_ERR_INV_DESCRIPTION] = "Invalid description",
  [SSERV_ERR_GROUP_CREATION_FAILED] = "Group creation failed",
  [SSERV_ERR_INV_SERVE_CONFIG_PATH] = "Invalid serve.cfg configuration file",
  [SSERV_ERR_INV_VARIANT] = "Invalid variant",
  [SSERV_ERR_UNSUPPORTED_SETTINGS] = "Settings of this contest are incompatible with this feature",
  [SSERV_ERR_INV_CNTS_SETTINGS] = "Invalid contest settings",
  [SSERV_ERR_INV_TEST_NUM] = "Invalid test number",
  [SSERV_ERR_FS_ERROR] = "Filesystem error",
  [SSERV_ERR_INV_EXIT_CODE] = "Invalid exit code",
  [SSERV_ERR_INV_SYS_GROUP] = "Invalid system group",
  [SSERV_ERR_INV_SYS_MODE] = "Invalid system mode",
  [SSERV_ERR_INV_TESTINFO] = "Invalid test information",
  [SSERV_ERR_INV_PROB_XML] = "Invalid statement XML file",
  [SSERV_ERR_UNSPEC_PROB_PACKAGE] = "Problem package is unspecified",
  [SSERV_ERR_UNSPEC_PROB_NAME] = "Problem name is unspecified",
  [SSERV_ERR_INV_XHTML] = "Invalid XHTML",

  "unknown error",
};

unsigned char const *
super_proto_strerror(int n)
{
  if (n < 0) n = -n;
  if (n >= SSERV_ERR_LAST) {
    // this is error anyway, so leak some memory
    unsigned char buf[64];

    snprintf(buf, sizeof(buf), "unknown error %d", n);
    return xstrdup(buf);
  }
  return super_proto_error_messages[n];
}

const int super_proto_op_redirect[SSERV_CMD_LAST] =
{
  [SSERV_CMD_USER_FILTER_FIRST_PAGE_ACTION] = SSERV_CMD_USER_FILTER_CHANGE_ACTION,
  [SSERV_CMD_USER_FILTER_PREV_PAGE_ACTION] = SSERV_CMD_USER_FILTER_CHANGE_ACTION,
  [SSERV_CMD_USER_FILTER_NEXT_PAGE_ACTION] = SSERV_CMD_USER_FILTER_CHANGE_ACTION,
  [SSERV_CMD_USER_FILTER_LAST_PAGE_ACTION] = SSERV_CMD_USER_FILTER_CHANGE_ACTION,

  [SSERV_CMD_USER_BROWSE_UNMARK_ALL_ACTION] = SSERV_CMD_USER_BROWSE_MARK_ALL_ACTION,
  [SSERV_CMD_USER_BROWSE_TOGGLE_ALL_ACTION] = SSERV_CMD_USER_BROWSE_MARK_ALL_ACTION,

  [SSERV_CMD_USER_SAVE_AND_PREV_ACTION] = SSERV_CMD_USER_SAVE_ACTION,
  [SSERV_CMD_USER_SAVE_AND_NEXT_ACTION] = SSERV_CMD_USER_SAVE_ACTION,

  [SSERV_CMD_USER_CANCEL_AND_PREV_ACTION] = SSERV_CMD_USER_CANCEL_ACTION,
  [SSERV_CMD_USER_CANCEL_AND_NEXT_ACTION] = SSERV_CMD_USER_CANCEL_ACTION,

  [SSERV_CMD_USER_SEL_CLEAR_CNTS_PASSWD_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_RANDOM_CNTS_PASSWD_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_DELETE_REG_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_CHANGE_REG_STATUS_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_CHANGE_REG_FLAGS_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_CREATE_REG_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_CREATE_REG_AND_COPY_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_CREATE_GROUP_MEMBER_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_DELETE_GROUP_MEMBER_PAGE] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE,

  [SSERV_CMD_USER_SEL_CLEAR_CNTS_PASSWD_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_RANDOM_CNTS_PASSWD_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_DELETE_REG_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_CHANGE_REG_STATUS_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_CHANGE_REG_FLAGS_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_CREATE_REG_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_CREATE_REG_AND_COPY_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_CREATE_GROUP_MEMBER_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,
  [SSERV_CMD_USER_SEL_DELETE_GROUP_MEMBER_ACTION] = SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION,

  [SSERV_CMD_USER_SEL_VIEW_CNTS_PASSWD_PAGE] = SSERV_CMD_USER_SEL_VIEW_PASSWD_PAGE,
  [SSERV_CMD_USER_SEL_VIEW_CNTS_PASSWD_REDIRECT] = SSERV_CMD_USER_SEL_VIEW_PASSWD_REDIRECT,

  [SSERV_CMD_GROUP_FILTER_FIRST_PAGE_ACTION] = SSERV_CMD_GROUP_FILTER_CHANGE_ACTION,
  [SSERV_CMD_GROUP_FILTER_PREV_PAGE_ACTION] = SSERV_CMD_GROUP_FILTER_CHANGE_ACTION,
  [SSERV_CMD_GROUP_FILTER_NEXT_PAGE_ACTION] = SSERV_CMD_GROUP_FILTER_CHANGE_ACTION,
  [SSERV_CMD_GROUP_FILTER_LAST_PAGE_ACTION] = SSERV_CMD_GROUP_FILTER_CHANGE_ACTION,

  [SSERV_CMD_TESTS_TEST_MOVE_DOWN_ACTION] = SSERV_CMD_TESTS_TEST_MOVE_UP_ACTION,
  [SSERV_CMD_TESTS_SAVED_MOVE_UP_ACTION] = SSERV_CMD_TESTS_TEST_MOVE_UP_ACTION,
  [SSERV_CMD_TESTS_SAVED_MOVE_DOWN_ACTION] = SSERV_CMD_TESTS_TEST_MOVE_UP_ACTION,
  [SSERV_CMD_TESTS_SAVED_MOVE_TO_TEST_ACTION] = SSERV_CMD_TESTS_TEST_MOVE_TO_SAVED_ACTION,

  [SSERV_CMD_TESTS_STATEMENT_EDIT_3_ACTION] = SSERV_CMD_TESTS_STATEMENT_EDIT_ACTION,
  [SSERV_CMD_TESTS_STATEMENT_EDIT_4_ACTION] = SSERV_CMD_TESTS_STATEMENT_EDIT_ACTION,
  [SSERV_CMD_TESTS_STATEMENT_DELETE_SAMPLE_ACTION] = SSERV_CMD_TESTS_STATEMENT_EDIT_ACTION,

  [SSERV_CMD_TESTS_STATEMENT_EDIT_5_ACTION] = SSERV_CMD_TESTS_STATEMENT_EDIT_2_ACTION,

  [SSERV_CMD_TESTS_SOURCE_FOOTER_EDIT_PAGE] = SSERV_CMD_TESTS_SOURCE_HEADER_EDIT_PAGE,
  [SSERV_CMD_TESTS_SOURCE_FOOTER_EDIT_ACTION] = SSERV_CMD_TESTS_SOURCE_HEADER_EDIT_ACTION,
  [SSERV_CMD_TESTS_SOURCE_FOOTER_DELETE_ACTION] = SSERV_CMD_TESTS_SOURCE_HEADER_DELETE_ACTION,
  [SSERV_CMD_TESTS_SOLUTION_EDIT_PAGE] = SSERV_CMD_TESTS_SOURCE_HEADER_EDIT_PAGE,
  [SSERV_CMD_TESTS_SOLUTION_EDIT_ACTION] = SSERV_CMD_TESTS_SOURCE_HEADER_EDIT_ACTION,
  [SSERV_CMD_TESTS_SOLUTION_DELETE_ACTION] = SSERV_CMD_TESTS_SOURCE_HEADER_DELETE_ACTION,

  [SSERV_CMD_TESTS_STYLE_CHECKER_CREATE_PAGE] = SSERV_CMD_TESTS_CHECKER_CREATE_PAGE,
  [SSERV_CMD_TESTS_VALUER_CREATE_PAGE] = SSERV_CMD_TESTS_CHECKER_CREATE_PAGE,
  [SSERV_CMD_TESTS_INTERACTOR_CREATE_PAGE] = SSERV_CMD_TESTS_CHECKER_CREATE_PAGE,
  [SSERV_CMD_TESTS_TEST_CHECKER_CREATE_PAGE] = SSERV_CMD_TESTS_CHECKER_CREATE_PAGE,
  [SSERV_CMD_TESTS_INIT_CREATE_PAGE] = SSERV_CMD_TESTS_CHECKER_CREATE_PAGE,

  [SSERV_CMD_TESTS_STYLE_CHECKER_CREATE_ACTION] = SSERV_CMD_TESTS_CHECKER_CREATE_ACTION,
  [SSERV_CMD_TESTS_VALUER_CREATE_ACTION] = SSERV_CMD_TESTS_CHECKER_CREATE_ACTION,
  [SSERV_CMD_TESTS_INTERACTOR_CREATE_ACTION] = SSERV_CMD_TESTS_CHECKER_CREATE_ACTION,
  [SSERV_CMD_TESTS_TEST_CHECKER_CREATE_ACTION] = SSERV_CMD_TESTS_CHECKER_CREATE_ACTION,
  [SSERV_CMD_TESTS_INIT_CREATE_ACTION] = SSERV_CMD_TESTS_CHECKER_CREATE_ACTION,

  [SSERV_CMD_TESTS_STYLE_CHECKER_EDIT_PAGE] = SSERV_CMD_TESTS_CHECKER_EDIT_PAGE,
  [SSERV_CMD_TESTS_VALUER_EDIT_PAGE] = SSERV_CMD_TESTS_CHECKER_EDIT_PAGE,
  [SSERV_CMD_TESTS_INTERACTOR_EDIT_PAGE] = SSERV_CMD_TESTS_CHECKER_EDIT_PAGE,
  [SSERV_CMD_TESTS_TEST_CHECKER_EDIT_PAGE] = SSERV_CMD_TESTS_CHECKER_EDIT_PAGE,
  [SSERV_CMD_TESTS_INIT_EDIT_PAGE] = SSERV_CMD_TESTS_CHECKER_EDIT_PAGE,

  [SSERV_CMD_TESTS_STYLE_CHECKER_EDIT_ACTION] = SSERV_CMD_TESTS_CHECKER_EDIT_ACTION,
  [SSERV_CMD_TESTS_VALUER_EDIT_ACTION] = SSERV_CMD_TESTS_CHECKER_EDIT_ACTION,
  [SSERV_CMD_TESTS_INTERACTOR_EDIT_ACTION] = SSERV_CMD_TESTS_CHECKER_EDIT_ACTION,
  [SSERV_CMD_TESTS_TEST_CHECKER_EDIT_ACTION] = SSERV_CMD_TESTS_CHECKER_EDIT_ACTION,
  [SSERV_CMD_TESTS_INIT_EDIT_ACTION] = SSERV_CMD_TESTS_CHECKER_EDIT_ACTION,

  [SSERV_CMD_TESTS_STYLE_CHECKER_DELETE_PAGE] = SSERV_CMD_TESTS_CHECKER_DELETE_PAGE,
  [SSERV_CMD_TESTS_VALUER_DELETE_PAGE] = SSERV_CMD_TESTS_CHECKER_DELETE_PAGE,
  [SSERV_CMD_TESTS_INTERACTOR_DELETE_PAGE] = SSERV_CMD_TESTS_CHECKER_DELETE_PAGE,
  [SSERV_CMD_TESTS_TEST_CHECKER_DELETE_PAGE] = SSERV_CMD_TESTS_CHECKER_DELETE_PAGE,
  [SSERV_CMD_TESTS_INIT_DELETE_PAGE] = SSERV_CMD_TESTS_CHECKER_DELETE_PAGE,

  [SSERV_CMD_TESTS_STYLE_CHECKER_DELETE_ACTION] = SSERV_CMD_TESTS_CHECKER_DELETE_ACTION,
  [SSERV_CMD_TESTS_VALUER_DELETE_ACTION] = SSERV_CMD_TESTS_CHECKER_DELETE_ACTION,
  [SSERV_CMD_TESTS_INTERACTOR_DELETE_ACTION] = SSERV_CMD_TESTS_CHECKER_DELETE_ACTION,
  [SSERV_CMD_TESTS_TEST_CHECKER_DELETE_ACTION] = SSERV_CMD_TESTS_CHECKER_DELETE_ACTION,
  [SSERV_CMD_TESTS_INIT_DELETE_ACTION] = SSERV_CMD_TESTS_CHECKER_DELETE_ACTION,

  [SSERV_CMD_TESTS_TEST_INSERT_PAGE] = SSERV_CMD_TESTS_TEST_EDIT_PAGE,
  [SSERV_CMD_TESTS_TEST_INSERT_ACTION] = SSERV_CMD_TESTS_TEST_EDIT_ACTION,

  [SSERV_CMD_TESTS_CANCEL_2_ACTION] = SSERV_CMD_TESTS_CANCEL_ACTION,

  [SSERV_CMD_TESTS_CHECK_TESTS_PAGE] = SSERV_CMD_TESTS_MAKE,
  [SSERV_CMD_TESTS_GENERATE_ANSWERS_PAGE] = SSERV_CMD_TESTS_MAKE,
  [SSERV_CMD_TESTS_TEST_GENERATE_ACTION] = SSERV_CMD_TESTS_TEST_CHECK_ACTION,

  [SSERV_CMD_DOWNLOAD_CLEANUP_AND_CHECK_ACTION] = SSERV_CMD_DOWNLOAD_CLEANUP_ACTION,
};

const unsigned char * const super_proto_cmd_names[SSERV_CMD_LAST] =
{
  [SSERV_CMD_PASS_FD] = "PASS_FD",
  [SSERV_CMD_STOP] = "STOP",
  [SSERV_CMD_RESTART] = "RESTART",
  [SSERV_CMD_OPEN_CONTEST] = "OPEN_CONTEST",
  [SSERV_CMD_CLOSE_CONTEST] = "CLOSE_CONTEST",
  [SSERV_CMD_INVISIBLE_CONTEST] = "INVISIBLE_CONTEST",
  [SSERV_CMD_VISIBLE_CONTEST] = "VISIBLE_CONTEST",
  [SSERV_CMD_SHOW_HIDDEN] = "SHOW_HIDDEN",
  [SSERV_CMD_HIDE_HIDDEN] = "HIDE_HIDDEN",
  [SSERV_CMD_SHOW_CLOSED] = "SHOW_CLOSED",
  [SSERV_CMD_HIDE_CLOSED] = "HIDE_CLOSED",
  [SSERV_CMD_SHOW_UNMNG] = "SHOW_UNMNG",
  [SSERV_CMD_HIDE_UNMNG] = "HIDE_UNMNG",
  [SSERV_CMD_RUN_LOG_TRUNC] = "RUN_LOG_TRUNC",
  [SSERV_CMD_RUN_LOG_DEV_NULL] = "RUN_LOG_DEV_NULL",
  [SSERV_CMD_RUN_LOG_FILE] = "RUN_LOG_FILE",
  [SSERV_CMD_RUN_MNG_TERM] = "RUN_MNG_TERM",
  [SSERV_CMD_CONTEST_RESTART] = "CONTEST_RESTART",
  [SSERV_CMD_CLEAR_MESSAGES] = "CLEAR_MESSAGES",
  [SSERV_CMD_RUN_MNG_RESET_ERROR] = "RUN_MNG_RESET_ERROR",
  [SSERV_CMD_CNTS_FORGET] = "CNTS_FORGET",
  [SSERV_CMD_CNTS_DEFAULT_ACCESS] = "CNTS_DEFAULT_ACCESS",
  [SSERV_CMD_CNTS_ADD_RULE] = "CNTS_ADD_RULE",
  [SSERV_CMD_CNTS_CHANGE_RULE] = "CNTS_CHANGE_RULE",
  [SSERV_CMD_CNTS_DELETE_RULE] = "CNTS_DELETE_RULE",
  [SSERV_CMD_CNTS_UP_RULE] = "CNTS_UP_RULE",
  [SSERV_CMD_CNTS_DOWN_RULE] = "CNTS_DOWN_RULE",
  [SSERV_CMD_CNTS_COPY_ACCESS] = "CNTS_COPY_ACCESS",
  [SSERV_CMD_CNTS_DELETE_PERMISSION] = "CNTS_DELETE_PERMISSION",
  [SSERV_CMD_CNTS_ADD_PERMISSION] = "CNTS_ADD_PERMISSION",
  [SSERV_CMD_CNTS_SAVE_PERMISSIONS] = "CNTS_SAVE_PERMISSIONS",
  [SSERV_CMD_CNTS_SET_PREDEF_PERMISSIONS] = "CNTS_SET_PREDEF_PERMISSIONS",
  [SSERV_CMD_CNTS_SAVE_FORM_FIELDS] = "CNTS_SAVE_FORM_FIELDS",
  [SSERV_CMD_CNTS_SAVE_CONTESTANT_FIELDS] = "CNTS_SAVE_CONTESTANT_FIELDS",
  [SSERV_CMD_CNTS_SAVE_RESERVE_FIELDS] = "CNTS_SAVE_RESERVE_FIELDS",
  [SSERV_CMD_CNTS_SAVE_COACH_FIELDS] = "CNTS_SAVE_COACH_FIELDS",
  [SSERV_CMD_CNTS_SAVE_ADVISOR_FIELDS] = "CNTS_SAVE_ADVISOR_FIELDS",
  [SSERV_CMD_CNTS_SAVE_GUEST_FIELDS] = "CNTS_SAVE_GUEST_FIELDS",
  [SSERV_CMD_LANG_SHOW_DETAILS] = "LANG_SHOW_DETAILS",
  [SSERV_CMD_LANG_HIDE_DETAILS] = "LANG_HIDE_DETAILS",
  [SSERV_CMD_LANG_DEACTIVATE] = "LANG_DEACTIVATE",
  [SSERV_CMD_LANG_ACTIVATE] = "LANG_ACTIVATE",
  [SSERV_CMD_PROB_ADD_ABSTRACT] = "PROB_ADD_ABSTRACT",
  [SSERV_CMD_PROB_ADD] = "PROB_ADD",
  [SSERV_CMD_PROB_SHOW_DETAILS] = "PROB_SHOW_DETAILS",
  [SSERV_CMD_PROB_HIDE_DETAILS] = "PROB_HIDE_DETAILS",
  [SSERV_CMD_PROB_SHOW_ADVANCED] = "PROB_SHOW_ADVANCED",
  [SSERV_CMD_PROB_HIDE_ADVANCED] = "PROB_HIDE_ADVANCED",
  [SSERV_CMD_PROB_DELETE] = "PROB_DELETE",
  [SSERV_CMD_PROB_CHANGE_VARIANTS] = "PROB_CHANGE_VARIANTS",
  [SSERV_CMD_PROB_DELETE_VARIANTS] = "PROB_DELETE_VARIANTS",
  [SSERV_CMD_LANG_UPDATE_VERSIONS] = "LANG_UPDATE_VERSIONS",
  [SSERV_CMD_PROB_CLEAR_VARIANTS] = "PROB_CLEAR_VARIANTS",
  [SSERV_CMD_PROB_RANDOM_VARIANTS] = "PROB_RANDOM_VARIANTS",
  [SSERV_CMD_LOGOUT] = "LOGOUT",
  [SSERV_CMD_HTTP_REQUEST] = "HTTP_REQUEST",
  [SSERV_CMD_EDITED_CNTS_BACK] = "EDITED_CNTS_BACK",
  [SSERV_CMD_EDITED_CNTS_CONTINUE] = "EDITED_CNTS_CONTINUE",
  [SSERV_CMD_EDITED_CNTS_START_NEW] = "EDITED_CNTS_START_NEW",
  [SSERV_CMD_LOCKED_CNTS_FORGET] = "LOCKED_CNTS_FORGET",
  [SSERV_CMD_LOCKED_CNTS_CONTINUE] = "LOCKED_CNTS_CONTINUE",
  [SSERV_CMD_USER_FILTER_CHANGE_ACTION] = "USER_FILTER_CHANGE_ACTION",
  [SSERV_CMD_USER_FILTER_FIRST_PAGE_ACTION] = "USER_FILTER_FIRST_PAGE_ACTION",
  [SSERV_CMD_USER_FILTER_PREV_PAGE_ACTION] = "USER_FILTER_PREV_PAGE_ACTION",
  [SSERV_CMD_USER_FILTER_NEXT_PAGE_ACTION] = "USER_FILTER_NEXT_PAGE_ACTION",
  [SSERV_CMD_USER_FILTER_LAST_PAGE_ACTION] = "USER_FILTER_LAST_PAGE_ACTION",
  [SSERV_CMD_USER_JUMP_CONTEST_ACTION] = "USER_JUMP_CONTEST_ACTION",
  [SSERV_CMD_USER_JUMP_GROUP_ACTION] = "USER_JUMP_GROUP_ACTION",
  [SSERV_CMD_USER_BROWSE_MARK_ALL_ACTION] = "USER_BROWSE_MARK_ALL_ACTION",
  [SSERV_CMD_USER_BROWSE_UNMARK_ALL_ACTION] = "USER_BROWSE_UNMARK_ALL_ACTION",
  [SSERV_CMD_USER_BROWSE_TOGGLE_ALL_ACTION] = "USER_BROWSE_TOGGLE_ALL_ACTION",
  [SSERV_CMD_USER_CREATE_ONE_PAGE] = "USER_CREATE_ONE_PAGE",
  [SSERV_CMD_USER_CREATE_ONE_ACTION] = "USER_CREATE_ONE_ACTION",
  [SSERV_CMD_USER_CREATE_MANY_PAGE] = "USER_CREATE_MANY_PAGE",
  [SSERV_CMD_USER_CREATE_MANY_ACTION] = "USER_CREATE_MANY_ACTION",
  [SSERV_CMD_USER_CREATE_FROM_CSV_PAGE] = "USER_CREATE_FROM_CSV_PAGE",
  [SSERV_CMD_USER_CREATE_FROM_CSV_ACTION] = "USER_CREATE_FROM_CSV_ACTION",
  [SSERV_CMD_USER_DETAIL_PAGE] = "USER_DETAIL_PAGE",
  [SSERV_CMD_USER_PASSWORD_PAGE] = "USER_PASSWORD_PAGE",
  [SSERV_CMD_USER_CHANGE_PASSWORD_ACTION] = "USER_CHANGE_PASSWORD_ACTION",
  [SSERV_CMD_USER_CNTS_PASSWORD_PAGE] = "USER_CNTS_PASSWORD_PAGE",
  [SSERV_CMD_USER_CHANGE_CNTS_PASSWORD_ACTION] = "USER_CHANGE_CNTS_PASSWORD_ACTION",
  [SSERV_CMD_USER_CLEAR_FIELD_ACTION] = "USER_CLEAR_FIELD_ACTION",
  [SSERV_CMD_USER_CREATE_MEMBER_ACTION] = "USER_CREATE_MEMBER_ACTION",
  [SSERV_CMD_USER_DELETE_MEMBER_PAGE] = "USER_DELETE_MEMBER_PAGE",
  [SSERV_CMD_USER_DELETE_MEMBER_ACTION] = "USER_DELETE_MEMBER_ACTION",
  [SSERV_CMD_USER_SAVE_AND_PREV_ACTION] = "USER_SAVE_AND_PREV_ACTION",
  [SSERV_CMD_USER_SAVE_ACTION] = "USER_SAVE_ACTION",
  [SSERV_CMD_USER_SAVE_AND_NEXT_ACTION] = "USER_SAVE_AND_NEXT_ACTION",
  [SSERV_CMD_USER_CANCEL_AND_PREV_ACTION] = "USER_CANCEL_AND_PREV_ACTION",
  [SSERV_CMD_USER_CANCEL_ACTION] = "USER_CANCEL_ACTION",
  [SSERV_CMD_USER_CANCEL_AND_NEXT_ACTION] = "USER_CANCEL_AND_NEXT_ACTION",
  [SSERV_CMD_USER_CREATE_REG_PAGE] = "USER_CREATE_REG_PAGE",
  [SSERV_CMD_USER_CREATE_REG_ACTION] = "USER_CREATE_REG_ACTION",
  [SSERV_CMD_USER_EDIT_REG_PAGE] = "USER_EDIT_REG_PAGE",
  [SSERV_CMD_USER_EDIT_REG_ACTION] = "USER_EDIT_REG_ACTION",
  [SSERV_CMD_USER_DELETE_REG_PAGE] = "USER_DELETE_REG_PAGE",
  [SSERV_CMD_USER_DELETE_REG_ACTION] = "USER_DELETE_REG_ACTION",
  [SSERV_CMD_USER_DELETE_SESSION_ACTION] = "USER_DELETE_SESSION_ACTION",
  [SSERV_CMD_USER_DELETE_ALL_SESSIONS_ACTION] = "USER_DELETE_ALL_SESSIONS_ACTION",
  [SSERV_CMD_USER_SEL_RANDOM_PASSWD_PAGE] = "USER_SEL_RANDOM_PASSWD_PAGE",
  [SSERV_CMD_USER_SEL_RANDOM_PASSWD_ACTION] = "USER_SEL_RANDOM_PASSWD_ACTION",
  [SSERV_CMD_USER_SEL_CLEAR_CNTS_PASSWD_PAGE] = "USER_SEL_CLEAR_CNTS_PASSWD_PAGE",
  [SSERV_CMD_USER_SEL_CLEAR_CNTS_PASSWD_ACTION] = "USER_SEL_CLEAR_CNTS_PASSWD_ACTION",
  [SSERV_CMD_USER_SEL_RANDOM_CNTS_PASSWD_PAGE] = "USER_SEL_RANDOM_CNTS_PASSWD_PAGE",
  [SSERV_CMD_USER_SEL_RANDOM_CNTS_PASSWD_ACTION] = "USER_SEL_RANDOM_CNTS_PASSWD_ACTION",
  [SSERV_CMD_USER_SEL_CREATE_REG_PAGE] = "USER_SEL_CREATE_REG_PAGE",
  [SSERV_CMD_USER_SEL_CREATE_REG_ACTION] = "USER_SEL_CREATE_REG_ACTION",
  [SSERV_CMD_USER_SEL_CREATE_REG_AND_COPY_PAGE] = "USER_SEL_CREATE_REG_AND_COPY_PAGE",
  [SSERV_CMD_USER_SEL_CREATE_REG_AND_COPY_ACTION] = "USER_SEL_CREATE_REG_AND_COPY_ACTION",
  [SSERV_CMD_USER_SEL_DELETE_REG_PAGE] = "USER_SEL_DELETE_REG_PAGE",
  [SSERV_CMD_USER_SEL_DELETE_REG_ACTION] = "USER_SEL_DELETE_REG_ACTION",
  [SSERV_CMD_USER_SEL_CHANGE_REG_STATUS_PAGE] = "USER_SEL_CHANGE_REG_STATUS_PAGE",
  [SSERV_CMD_USER_SEL_CHANGE_REG_STATUS_ACTION] = "USER_SEL_CHANGE_REG_STATUS_ACTION",
  [SSERV_CMD_USER_SEL_CHANGE_REG_FLAGS_PAGE] = "USER_SEL_CHANGE_REG_FLAGS_PAGE",
  [SSERV_CMD_USER_SEL_CHANGE_REG_FLAGS_ACTION] = "USER_SEL_CHANGE_REG_FLAGS_ACTION",
  [SSERV_CMD_USER_SEL_CANCEL_ACTION] = "USER_SEL_CANCEL_ACTION",
  [SSERV_CMD_USER_SEL_VIEW_PASSWD_PAGE] = "USER_SEL_VIEW_PASSWD_PAGE",
  [SSERV_CMD_USER_SEL_VIEW_CNTS_PASSWD_PAGE] = "USER_SEL_VIEW_CNTS_PASSWD_PAGE",
  [SSERV_CMD_USER_SEL_VIEW_PASSWD_REDIRECT] = "USER_SEL_VIEW_PASSWD_REDIRECT",
  [SSERV_CMD_USER_SEL_VIEW_CNTS_PASSWD_REDIRECT] = "USER_SEL_VIEW_CNTS_PASSWD_REDIRECT",
  [SSERV_CMD_USER_SEL_CREATE_GROUP_MEMBER_PAGE] = "USER_SEL_CREATE_GROUP_MEMBER_PAGE",
  [SSERV_CMD_USER_SEL_CREATE_GROUP_MEMBER_ACTION] = "USER_SEL_CREATE_GROUP_MEMBER_ACTION",
  [SSERV_CMD_USER_SEL_DELETE_GROUP_MEMBER_PAGE] = "USER_SEL_DELETE_GROUP_MEMBER_PAGE",
  [SSERV_CMD_USER_SEL_DELETE_GROUP_MEMBER_ACTION] = "USER_SEL_DELETE_GROUP_MEMBER_ACTION",
  [SSERV_CMD_USER_IMPORT_CSV_PAGE] = "USER_IMPORT_CSV_PAGE",
  [SSERV_CMD_USER_IMPORT_CSV_ACTION] = "USER_IMPORT_CSV_ACTION",
  [SSERV_CMD_GROUP_BROWSE_PAGE] = "GROUP_BROWSE_PAGE",
  [SSERV_CMD_GROUP_FILTER_CHANGE_ACTION] = "GROUP_FILTER_CHANGE_ACTION",
  [SSERV_CMD_GROUP_FILTER_FIRST_PAGE_ACTION] = "GROUP_FILTER_FIRST_PAGE_ACTION",
  [SSERV_CMD_GROUP_FILTER_PREV_PAGE_ACTION] = "GROUP_FILTER_PREV_PAGE_ACTION",
  [SSERV_CMD_GROUP_FILTER_NEXT_PAGE_ACTION] = "GROUP_FILTER_NEXT_PAGE_ACTION",
  [SSERV_CMD_GROUP_FILTER_LAST_PAGE_ACTION] = "GROUP_FILTER_LAST_PAGE_ACTION",
  [SSERV_CMD_GROUP_CREATE_PAGE] = "GROUP_CREATE_PAGE",
  [SSERV_CMD_GROUP_CREATE_ACTION] = "GROUP_CREATE_ACTION",
  [SSERV_CMD_GROUP_MODIFY_PAGE] = "GROUP_MODIFY_PAGE",
  [SSERV_CMD_GROUP_MODIFY_PAGE_ACTION] = "GROUP_MODIFY_PAGE_ACTION",
  [SSERV_CMD_GROUP_MODIFY_ACTION] = "GROUP_MODIFY_ACTION",
  [SSERV_CMD_GROUP_DELETE_PAGE] = "GROUP_DELETE_PAGE",
  [SSERV_CMD_GROUP_DELETE_PAGE_ACTION] = "GROUP_DELETE_PAGE_ACTION",
  [SSERV_CMD_GROUP_DELETE_ACTION] = "GROUP_DELETE_ACTION",
  [SSERV_CMD_GROUP_CANCEL_ACTION] = "GROUP_CANCEL_ACTION",
  [SSERV_CMD_TESTS_MAIN_PAGE] = "TESTS_MAIN_PAGE",
  [SSERV_CMD_TESTS_STATEMENT_EDIT_PAGE] = "TESTS_STATEMENT_EDIT_PAGE",
  [SSERV_CMD_TESTS_STATEMENT_EDIT_ACTION] = "TESTS_STATEMENT_EDIT_ACTION",
  [SSERV_CMD_TESTS_STATEMENT_EDIT_2_ACTION] = "TESTS_STATEMENT_EDIT_2_ACTION",
  [SSERV_CMD_TESTS_STATEMENT_EDIT_3_ACTION] = "TESTS_STATEMENT_EDIT_3_ACTION",
  [SSERV_CMD_TESTS_STATEMENT_EDIT_4_ACTION] = "TESTS_STATEMENT_EDIT_4_ACTION",
  [SSERV_CMD_TESTS_STATEMENT_EDIT_5_ACTION] = "TESTS_STATEMENT_EDIT_5_ACTION",
  [SSERV_CMD_TESTS_STATEMENT_DELETE_ACTION] = "TESTS_STATEMENT_DELETE_ACTION",
  [SSERV_CMD_TESTS_STATEMENT_DELETE_SAMPLE_ACTION] = "TESTS_STATEMENT_DELETE_SAMPLE_ACTION",
  [SSERV_CMD_TESTS_TESTS_VIEW_PAGE] = "TESTS_TESTS_VIEW_PAGE",
  [SSERV_CMD_TESTS_CHECK_TESTS_PAGE] = "TESTS_CHECK_TESTS_PAGE",
  [SSERV_CMD_TESTS_GENERATE_ANSWERS_PAGE] = "TESTS_GENERATE_ANSWERS_PAGE",
  [SSERV_CMD_TESTS_SOURCE_HEADER_EDIT_PAGE] = "TESTS_SOURCE_HEADER_EDIT_PAGE",
  [SSERV_CMD_TESTS_SOURCE_HEADER_EDIT_ACTION] = "TESTS_SOURCE_HEADER_EDIT_ACTION",
  [SSERV_CMD_TESTS_SOURCE_HEADER_DELETE_ACTION] = "TESTS_SOURCE_HEADER_DELETE_ACTION",
  [SSERV_CMD_TESTS_SOURCE_FOOTER_EDIT_PAGE] = "TESTS_SOURCE_FOOTER_EDIT_PAGE",
  [SSERV_CMD_TESTS_SOURCE_FOOTER_EDIT_ACTION] = "TESTS_SOURCE_FOOTER_EDIT_ACTION",
  [SSERV_CMD_TESTS_SOURCE_FOOTER_DELETE_ACTION] = "TESTS_SOURCE_FOOTER_DELETE_ACTION",
  [SSERV_CMD_TESTS_SOLUTION_EDIT_PAGE] = "TESTS_SOLUTION_EDIT_PAGE",
  [SSERV_CMD_TESTS_SOLUTION_EDIT_ACTION] = "TESTS_SOLUTION_EDIT_ACTION",
  [SSERV_CMD_TESTS_SOLUTION_DELETE_ACTION] = "TESTS_SOLUTION_DELETE_ACTION",
  [SSERV_CMD_TESTS_STYLE_CHECKER_CREATE_PAGE] = "TESTS_STYLE_CHECKER_CREATE_PAGE",
  [SSERV_CMD_TESTS_STYLE_CHECKER_CREATE_ACTION] = "TESTS_STYLE_CHECKER_CREATE_ACTION",
  [SSERV_CMD_TESTS_STYLE_CHECKER_EDIT_PAGE] = "TESTS_STYLE_CHECKER_EDIT_PAGE",
  [SSERV_CMD_TESTS_STYLE_CHECKER_EDIT_ACTION] = "TESTS_STYLE_CHECKER_EDIT_ACTION",
  [SSERV_CMD_TESTS_STYLE_CHECKER_DELETE_PAGE] = "TESTS_STYLE_CHECKER_DELETE_PAGE",
  [SSERV_CMD_TESTS_STYLE_CHECKER_DELETE_ACTION] = "TESTS_STYLE_CHECKER_DELETE_ACTION",
  [SSERV_CMD_TESTS_CHECKER_CREATE_PAGE] = "TESTS_CHECKER_CREATE_PAGE",
  [SSERV_CMD_TESTS_CHECKER_CREATE_ACTION] = "TESTS_CHECKER_CREATE_ACTION",
  [SSERV_CMD_TESTS_CHECKER_EDIT_PAGE] = "TESTS_CHECKER_EDIT_PAGE",
  [SSERV_CMD_TESTS_CHECKER_EDIT_ACTION] = "TESTS_CHECKER_EDIT_ACTION",
  [SSERV_CMD_TESTS_CHECKER_DELETE_PAGE] = "TESTS_CHECKER_DELETE_PAGE",
  [SSERV_CMD_TESTS_CHECKER_DELETE_ACTION] = "TESTS_CHECKER_DELETE_ACTION",
  [SSERV_CMD_TESTS_VALUER_CREATE_PAGE] = "TESTS_VALUER_CREATE_PAGE",
  [SSERV_CMD_TESTS_VALUER_CREATE_ACTION] = "TESTS_VALUER_CREATE_ACTION",
  [SSERV_CMD_TESTS_VALUER_EDIT_PAGE] = "TESTS_VALUER_EDIT_PAGE",
  [SSERV_CMD_TESTS_VALUER_EDIT_ACTION] = "TESTS_VALUER_EDIT_ACTION",
  [SSERV_CMD_TESTS_VALUER_DELETE_PAGE] = "TESTS_VALUER_DELETE_PAGE",
  [SSERV_CMD_TESTS_VALUER_DELETE_ACTION] = "TESTS_VALUER_DELETE_ACTION",
  [SSERV_CMD_TESTS_INTERACTOR_CREATE_PAGE] = "TESTS_INTERACTOR_CREATE_PAGE",
  [SSERV_CMD_TESTS_INTERACTOR_CREATE_ACTION] = "TESTS_INTERACTOR_CREATE_ACTION",
  [SSERV_CMD_TESTS_INTERACTOR_EDIT_PAGE] = "TESTS_INTERACTOR_EDIT_PAGE",
  [SSERV_CMD_TESTS_INTERACTOR_EDIT_ACTION] = "TESTS_INTERACTOR_EDIT_ACTION",
  [SSERV_CMD_TESTS_INTERACTOR_DELETE_PAGE] = "TESTS_INTERACTOR_DELETE_PAGE",
  [SSERV_CMD_TESTS_INTERACTOR_DELETE_ACTION] = "TESTS_INTERACTOR_DELETE_ACTION",
  [SSERV_CMD_TESTS_TEST_CHECKER_CREATE_PAGE] = "TESTS_TEST_CHECKER_CREATE_PAGE",
  [SSERV_CMD_TESTS_TEST_CHECKER_CREATE_ACTION] = "TESTS_TEST_CHECKER_CREATE_ACTION",
  [SSERV_CMD_TESTS_TEST_CHECKER_EDIT_PAGE] = "TESTS_TEST_CHECKER_EDIT_PAGE",
  [SSERV_CMD_TESTS_TEST_CHECKER_EDIT_ACTION] = "TESTS_TEST_CHECKER_EDIT_ACTION",
  [SSERV_CMD_TESTS_TEST_CHECKER_DELETE_PAGE] = "TESTS_TEST_CHECKER_DELETE_PAGE",
  [SSERV_CMD_TESTS_TEST_CHECKER_DELETE_ACTION] = "TESTS_TEST_CHECKER_DELETE_ACTION",
  [SSERV_CMD_TESTS_INIT_CREATE_PAGE] = "TESTS_INIT_CREATE_PAGE",
  [SSERV_CMD_TESTS_INIT_CREATE_ACTION] = "TESTS_INIT_CREATE_ACTION",
  [SSERV_CMD_TESTS_INIT_EDIT_PAGE] = "TESTS_INIT_EDIT_PAGE",
  [SSERV_CMD_TESTS_INIT_EDIT_ACTION] = "TESTS_INIT_EDIT_ACTION",
  [SSERV_CMD_TESTS_INIT_DELETE_PAGE] = "TESTS_INIT_DELETE_PAGE",
  [SSERV_CMD_TESTS_INIT_DELETE_ACTION] = "TESTS_INIT_DELETE_ACTION",
  [SSERV_CMD_TESTS_MAKEFILE_EDIT_PAGE] = "TESTS_MAKEFILE_EDIT_PAGE",
  [SSERV_CMD_TESTS_MAKEFILE_EDIT_ACTION] = "TESTS_MAKEFILE_EDIT_ACTION",
  [SSERV_CMD_TESTS_MAKEFILE_DELETE_ACTION] = "TESTS_MAKEFILE_DELETE_ACTION",
  [SSERV_CMD_TESTS_MAKEFILE_GENERATE_ACTION] = "TESTS_MAKEFILE_GENERATE_ACTION",
  [SSERV_CMD_TESTS_TEST_MOVE_UP_ACTION] = "TESTS_TEST_MOVE_UP_ACTION",
  [SSERV_CMD_TESTS_TEST_MOVE_DOWN_ACTION] = "TESTS_TEST_MOVE_DOWN_ACTION",
  [SSERV_CMD_TESTS_TEST_MOVE_TO_SAVED_ACTION] = "TESTS_TEST_MOVE_TO_SAVED_ACTION",
  [SSERV_CMD_TESTS_TEST_INSERT_PAGE] = "TESTS_TEST_INSERT_PAGE",
  [SSERV_CMD_TESTS_TEST_INSERT_ACTION] = "TESTS_TEST_INSERT_ACTION",
  [SSERV_CMD_TESTS_TEST_EDIT_PAGE] = "TESTS_TEST_EDIT_PAGE",
  [SSERV_CMD_TESTS_TEST_EDIT_ACTION] = "TESTS_TEST_EDIT_ACTION",
  [SSERV_CMD_TESTS_TEST_DELETE_PAGE] = "TESTS_TEST_DELETE_PAGE",
  [SSERV_CMD_TESTS_TEST_DELETE_ACTION] = "TESTS_TEST_DELETE_ACTION",
  [SSERV_CMD_TESTS_TEST_UPLOAD_ARCHIVE_1_PAGE] = "TESTS_TEST_UPLOAD_ARCHIVE_1_PAGE",
  [SSERV_CMD_TESTS_TEST_CHECK_ACTION] = "TESTS_TEST_CHECK_ACTION",
  [SSERV_CMD_TESTS_TEST_GENERATE_ACTION] = "TESTS_TEST_GENERATE_ACTION",
  [SSERV_CMD_TESTS_SAVED_MOVE_UP_ACTION] = "TESTS_SAVED_MOVE_UP_ACTION",
  [SSERV_CMD_TESTS_SAVED_MOVE_DOWN_ACTION] = "TESTS_SAVED_MOVE_DOWN_ACTION",
  [SSERV_CMD_TESTS_SAVED_DELETE_PAGE] = "TESTS_SAVED_DELETE_PAGE",
  [SSERV_CMD_TESTS_SAVED_MOVE_TO_TEST_ACTION] = "TESTS_SAVED_MOVE_TO_TEST_ACTION",
  [SSERV_CMD_TESTS_README_CREATE_PAGE] = "TESTS_README_CREATE_PAGE",
  [SSERV_CMD_TESTS_README_EDIT_PAGE] = "TESTS_README_EDIT_PAGE",
  [SSERV_CMD_TESTS_README_DELETE_PAGE] = "TESTS_README_DELETE_PAGE",
  [SSERV_CMD_TESTS_CANCEL_ACTION] = "TESTS_CANCEL_ACTION",
  [SSERV_CMD_TESTS_CANCEL_2_ACTION] = "TESTS_CANCEL_2_ACTION",
  [SSERV_CMD_TESTS_TEST_DOWNLOAD] = "TESTS_TEST_DOWNLOAD",
  [SSERV_CMD_TESTS_TEST_UPLOAD_PAGE] = "TESTS_TEST_UPLOAD_PAGE",
  [SSERV_CMD_TESTS_TEST_CLEAR_INF_ACTION] = "TESTS_TEST_CLEAR_INF_ACTION",
  [SSERV_CMD_TESTS_MAKE] = "TESTS_MAKE",
  [SSERV_CMD_USER_MAP_MAIN_PAGE] = "USER_MAP_MAIN_PAGE",
  [SSERV_CMD_USER_MAP_DELETE_ACTION] = "USER_MAP_DELETE_ACTION",
  [SSERV_CMD_USER_MAP_ADD_ACTION] = "USER_MAP_ADD_ACTION",
  [SSERV_CMD_CAPS_MAIN_PAGE] = "CAPS_MAIN_PAGE",
  [SSERV_CMD_CAPS_DELETE_ACTION] = "CAPS_DELETE_ACTION",
  [SSERV_CMD_CAPS_ADD_ACTION] = "CAPS_ADD_ACTION",
  [SSERV_CMD_CAPS_EDIT_PAGE] = "CAPS_EDIT_PAGE",
  [SSERV_CMD_CAPS_EDIT_SAVE_ACTION] = "CAPS_EDIT_SAVE_ACTION",
  [SSERV_CMD_CAPS_EDIT_CANCEL_ACTION] = "CAPS_EDIT_CANCEL_ACTION",
  [SSERV_CMD_EJUDGE_XML_UPDATE_ACTION] = "EJUDGE_XML_UPDATE_ACTION",
  [SSERV_CMD_EJUDGE_XML_CANCEL_ACTION] = "EJUDGE_XML_CANCEL_ACTION",
  [SSERV_CMD_EJUDGE_XML_MUST_RESTART] = "EJUDGE_XML_MUST_RESTART",
  [SSERV_CMD_IMPORT_FROM_POLYGON_PAGE] = "IMPORT_FROM_POLYGON_PAGE",
  [SSERV_CMD_IMPORT_FROM_POLYGON_ACTION] = "IMPORT_FROM_POLYGON_ACTION",
  [SSERV_CMD_DOWNLOAD_PROGRESS_PAGE] = "DOWNLOAD_PROGRESS_PAGE",
  [SSERV_CMD_DOWNLOAD_CLEANUP_ACTION] = "DOWNLOAD_CLEANUP_ACTION",
  [SSERV_CMD_DOWNLOAD_KILL_ACTION] = "DOWNLOAD_KILL_ACTION",
  [SSERV_CMD_DOWNLOAD_CLEANUP_AND_CHECK_ACTION] = "DOWNLOAD_CLEANUP_AND_CHECK_ACTION",
  [SSERV_CMD_DOWNLOAD_CLEANUP_AND_IMPORT_ACTION] = "DOWNLOAD_CLEANUP_AND_IMPORT_ACTION",
  [SSERV_CMD_UPDATE_FROM_POLYGON_PAGE] = "UPDATE_FROM_POLYGON_PAGE",
  [SSERV_CMD_UPDATE_FROM_POLYGON_ACTION] = "UPDATE_FROM_POLYGON_ACTION",
  [SSERV_CMD_IMPORT_PROBLEMS_BATCH_ACTION] = "IMPORT_PROBLEMS_BATCH_ACTION",
  [SSERV_CMD_CREATE_CONTEST_BATCH_ACTION] = "CREATE_CONTEST_BATCH_ACTION",
  [SSERV_CMD_IMPORT_CONTEST_FROM_POLYGON_PAGE] = "IMPORT_CONTEST_FROM_POLYGON_PAGE",
  [SSERV_CMD_LOGIN_PAGE] = "LOGIN_PAGE",
  [SSERV_CMD_MAIN_PAGE] = "MAIN_PAGE",
  [SSERV_CMD_CONTEST_PAGE] = "CONTEST_PAGE",
  [SSERV_CMD_CONTEST_XML_PAGE] = "CONTEST_XML_PAGE",
  [SSERV_CMD_SERVE_CFG_PAGE] = "SERVE_CFG_PAGE",
  [SSERV_CMD_CREATE_CONTEST_PAGE] = "CREATE_CONTEST_PAGE",
  [SSERV_CMD_CREATE_CONTEST_2_ACTION] = "CREATE_CONTEST_2_ACTION",
  [SSERV_CMD_CONTEST_ALREADY_EDITED_PAGE] = "CONTEST_ALREADY_EDITED_PAGE",
  [SSERV_CMD_CONTEST_LOCKED_PAGE] = "CONTEST_LOCKED_PAGE",
  [SSERV_CMD_CHECK_TESTS_PAGE] = "CHECK_TESTS_PAGE",
  [SSERV_CMD_CNTS_EDIT_PERMISSIONS_PAGE] = "CNTS_EDIT_PERMISSIONS_PAGE",
  [SSERV_CMD_CNTS_EDIT_REGISTER_ACCESS_PAGE] = "CNTS_EDIT_REGISTER_ACCESS_PAGE",
  [SSERV_CMD_CNTS_EDIT_USERS_ACCESS_PAGE] = "CNTS_EDIT_USERS_ACCESS_PAGE",
  [SSERV_CMD_CNTS_EDIT_MASTER_ACCESS_PAGE] = "CNTS_EDIT_MASTER_ACCESS_PAGE",
  [SSERV_CMD_CNTS_EDIT_JUDGE_ACCESS_PAGE] = "CNTS_EDIT_JUDGE_ACCESS_PAGE",
  [SSERV_CMD_CNTS_EDIT_TEAM_ACCESS_PAGE] = "CNTS_EDIT_TEAM_ACCESS_PAGE",
  [SSERV_CMD_CNTS_EDIT_SERVE_CONTROL_ACCESS_PAGE] = "CNTS_EDIT_SERVE_CONTROL_ACCESS_PAGE",
  [SSERV_CMD_CNTS_EDIT_USER_FIELDS_PAGE] = "CNTS_EDIT_USER_FIELDS_PAGE",
  [SSERV_CMD_CNTS_EDIT_CONTESTANT_FIELDS_PAGE] = "CNTS_EDIT_CONTESTANT_FIELDS_PAGE",
  [SSERV_CMD_CNTS_EDIT_RESERVE_FIELDS_PAGE] = "CNTS_EDIT_RESERVE_FIELDS_PAGE",
  [SSERV_CMD_CNTS_EDIT_COACH_FIELDS_PAGE] = "CNTS_EDIT_COACH_FIELDS_PAGE",
  [SSERV_CMD_CNTS_EDIT_ADVISOR_FIELDS_PAGE] = "CNTS_EDIT_ADVISOR_FIELDS_PAGE",
  [SSERV_CMD_CNTS_EDIT_GUEST_FIELDS_PAGE] = "CNTS_EDIT_GUEST_FIELDS_PAGE",
  [SSERV_CMD_CNTS_START_EDIT_ACTION] = "CNTS_START_EDIT_ACTION",
  [SSERV_CMD_CNTS_EDIT_CUR_CONTEST_PAGE] = "CNTS_EDIT_CUR_CONTEST_PAGE",
  [SSERV_CMD_GLOB_EDIT_CONTEST_START_CMD_PAGE] = "GLOB_EDIT_CONTEST_START_CMD_PAGE",
  [SSERV_CMD_GLOB_EDIT_CONTEST_STOP_CMD_PAGE] = "GLOB_EDIT_CONTEST_STOP_CMD_PAGE",
  [SSERV_CMD_GLOB_EDIT_STAND_HEADER_FILE_PAGE] = "GLOB_EDIT_STAND_HEADER_FILE_PAGE",
  [SSERV_CMD_GLOB_EDIT_STAND_FOOTER_FILE_PAGE] = "GLOB_EDIT_STAND_FOOTER_FILE_PAGE",
  [SSERV_CMD_GLOB_EDIT_STAND2_HEADER_FILE_PAGE] = "GLOB_EDIT_STAND2_HEADER_FILE_PAGE",
  [SSERV_CMD_GLOB_EDIT_STAND2_FOOTER_FILE_PAGE] = "GLOB_EDIT_STAND2_FOOTER_FILE_PAGE",
  [SSERV_CMD_GLOB_EDIT_PLOG_HEADER_FILE_PAGE] = "GLOB_EDIT_PLOG_HEADER_FILE_PAGE",
  [SSERV_CMD_GLOB_EDIT_PLOG_FOOTER_FILE_PAGE] = "GLOB_EDIT_PLOG_FOOTER_FILE_PAGE",
  [SSERV_CMD_CNTS_EDIT_USERS_HEADER_PAGE] = "CNTS_EDIT_USERS_HEADER_PAGE",
  [SSERV_CMD_CNTS_EDIT_USERS_FOOTER_PAGE] = "CNTS_EDIT_USERS_FOOTER_PAGE",
  [SSERV_CMD_CNTS_EDIT_COPYRIGHT_PAGE] = "CNTS_EDIT_COPYRIGHT_PAGE",
  [SSERV_CMD_CNTS_EDIT_WELCOME_PAGE] = "CNTS_EDIT_WELCOME_PAGE",
  [SSERV_CMD_CNTS_EDIT_REG_WELCOME_PAGE] = "CNTS_EDIT_REG_WELCOME_PAGE",
  [SSERV_CMD_CNTS_EDIT_REGISTER_EMAIL_FILE_PAGE] = "CNTS_EDIT_REGISTER_EMAIL_FILE_PAGE",
  [SSERV_CMD_CNTS_RELOAD_FILE_ACTION] = "CNTS_RELOAD_FILE_ACTION",
  [SSERV_CMD_CNTS_SAVE_FILE_ACTION] = "CNTS_SAVE_FILE_ACTION",
  [SSERV_CMD_CNTS_CLEAR_FILE_ACTION] = "CNTS_CLEAR_FILE_ACTION",
  [SSERV_CMD_CNTS_EDIT_CUR_GLOBAL_PAGE] = "CNTS_EDIT_CUR_GLOBAL_PAGE",
  [SSERV_CMD_CNTS_EDIT_CUR_LANGUAGES_PAGE] = "CNTS_EDIT_CUR_LANGUAGES_PAGE",
  [SSERV_CMD_CNTS_EDIT_CUR_PROBLEMS_PAGE] = "CNTS_EDIT_CUR_PROBLEMS_PAGE",
  [SSERV_CMD_CNTS_EDIT_CUR_PROBLEM_PAGE] = "CNTS_EDIT_CUR_PROBLEM_PAGE",
  [SSERV_CMD_CNTS_START_EDIT_PROBLEM_ACTION] = "CNTS_START_EDIT_PROBLEM_ACTION",
  [SSERV_CMD_CNTS_START_EDIT_VARIANT_ACTION] = "CNTS_START_EDIT_VARIANT_ACTION",
  [SSERV_CMD_CNTS_EDIT_CUR_VARIANT_PAGE] = "CNTS_EDIT_CUR_VARIANT_PAGE",
  [SSERV_CMD_CNTS_NEW_SERVE_CFG_PAGE] = "CNTS_NEW_SERVE_CFG_PAGE",
  [SSERV_CMD_CNTS_COMMIT_PAGE] = "CNTS_COMMIT_PAGE",

  [SSERV_CMD_USER_BROWSE_PAGE] = "USER_BROWSE_PAGE",
  [SSERV_CMD_USER_BROWSE_DATA] = "USER_BROWSE_DATA",
  [SSERV_CMD_GET_CONTEST_LIST] = "GET_CONTEST_LIST",
  [SSERV_CMD_CNTS_SAVE_BASIC_FORM] = "CNTS_SAVE_BASIC_FORM",
  [SSERV_CMD_CNTS_SAVE_FLAGS_FORM] = "CNTS_SAVE_FLAGS_FORM",
  [SSERV_CMD_CNTS_SAVE_REGISTRATION_FORM] = "CNTS_SAVE_REGISTRATION_FORM",
  [SSERV_CMD_CNTS_SAVE_TIMING_FORM] = "CNTS_SAVE_TIMING_FORM",
  [SSERV_CMD_CNTS_SAVE_URLS_FORM] = "CNTS_SAVE_URLS_FORM",
  [SSERV_CMD_CNTS_SAVE_HEADERS_FORM] = "CNTS_SAVE_HEADERS_FORM",
  [SSERV_CMD_CNTS_SAVE_ATTRS_FORM] = "CNTS_SAVE_ATTRS_FORM",
  [SSERV_CMD_CNTS_SAVE_NOTIFICATIONS_FORM] = "CNTS_SAVE_NOTIFICATIONS_FORM",
  [SSERV_CMD_CNTS_SAVE_ADVANCED_FORM] = "CNTS_SAVE_ADVANCED_FORM",

  [SSERV_CMD_GLOB_SAVE_MAIN_FORM] = "GLOB_SAVE_MAIN_FORM",
  [SSERV_CMD_GLOB_SAVE_CAPABILITIES_FORM] = "GLOB_SAVE_CAPABILITIES_FORM",
  [SSERV_CMD_GLOB_SAVE_FILES_FORM] = "GLOB_SAVE_FILES_FORM",
  [SSERV_CMD_GLOB_SAVE_QUOTAS_FORM] = "GLOB_SAVE_QUOTAS_FORM",
  [SSERV_CMD_GLOB_SAVE_URLS_FORM] = "GLOB_SAVE_URLS_FORM",
  [SSERV_CMD_GLOB_SAVE_ATTRS_FORM] = "GLOB_SAVE_ATTRS_FORM",
  [SSERV_CMD_GLOB_SAVE_ADVANCED_FORM] = "GLOB_SAVE_ADVANCED_FORM",
  [SSERV_CMD_GLOB_SAVE_LIMITS_FORM] = "GLOB_SAVE_LIMITS_FORM",
  [SSERV_CMD_LANG_SAVE_MAIN_FORM] = "LANG_SAVE_MAIN_FORM",
  [SSERV_CMD_PROB_SAVE_ID_FORM] = "PROB_SAVE_ID_FORM",
  [SSERV_CMD_PROB_SAVE_FILES_FORM] = "PROB_SAVE_FILES_FORM",
  [SSERV_CMD_PROB_SAVE_VALIDATION_FORM] = "PROB_SAVE_VALIDATION_FORM",
  [SSERV_CMD_PROB_SAVE_VIEW_FORM] = "PROB_SAVE_VIEW_FORM",
  [SSERV_CMD_PROB_SAVE_SUBMISSION_FORM] = "PROB_SAVE_SUBMISSION_FORM",
  [SSERV_CMD_PROB_SAVE_COMPILING_FORM] = "PROB_SAVE_COMPILING_FORM",
  [SSERV_CMD_PROB_SAVE_RUNNING_FORM] = "PROB_SAVE_RUNNING_FORM",
  [SSERV_CMD_PROB_SAVE_LIMITS_FORM] = "PROB_SAVE_LIMITS_FORM",
  [SSERV_CMD_PROB_SAVE_CHECKING_FORM] = "PROB_SAVE_CHECKING_FORM",
  [SSERV_CMD_PROB_SAVE_SCORING_FORM] = "PROB_SAVE_SCORING_FORM",
  [SSERV_CMD_PROB_SAVE_FEEDBACK_FORM] = "PROB_SAVE_FEEDBACK_FORM",
  [SSERV_CMD_PROB_SAVE_STANDING_FORM] = "PROB_SAVE_STANDING_FORM",

  [SSERV_CMD_MAIN_PAGE_BUTTON] = "MAIN_PAGE_BUTTON",

  [SSERV_CMD_MIGRATION_PAGE] = "MIGRATION_PAGE",
  [SSERV_CMD_EDIT_SESSIONS_PAGE] = "EDIT_SESSIONS_PAGE",
  [SSERV_CMD_CLEAR_SESSION] = "CLEAR_SESSION",
  [SSERV_CMD_LANG_AJAX_ACTION] = "LANG_AJAX_ACTION",
  [SSERV_CMD_LOGIN_ACTION_JSON] = "LOGIN_ACTION_JSON",
};
